<?php
namespace Modules\DF\Factories;

use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

use Fiscal\Entities\Notafiscal;
use Geral\Entities\Empresa;
use Geral\Entities\Endeforn;
use Geral\Entities\Endeclie;
use Geral\Entities\Endefili;
use Estoque\Entities\Lanctoestoq;

class Lancamento{
    
    const PROCEDURE_GERA_PRODUTO            = 'GERA_NOTAFISCAL_PRODUTO';
    const PROCEDURE_GERA_FECHAMENTO_SAIDA   = 'GERA_NOTAFISCAL_FECHAMENTO';
    const PROCEDURE_GERA_FECHAMENTO_ENTRADA = 'GERA_FECHAMENTO_NFENTRADA';
    const PROCEDURE_GERA_NOTA_CORPO_ENTRADA = 'GERA_NOTAFISCAL_CORPO_ENTRADA';
    const PROCEDURE_GERA_NOTA_CORPO         = 'GERA_NOTAFISCAL_CORPO';
    const PROCEDURE_IMPOSTO_ENTRADA_PRODUTO = 'IMPOSTO_ENTRADA_PRODUTO';
    const PROCEDURE_CONSULTA_GRUPOFISCAL    = 'CONSULTA_GRUPOFISCAL';
    const PROCEDURE_CONFERE_ERROS_NF        = 'CONFERE_ERROS_NF';
    const PROCEDURE_NF_BAIXA_AF             = 'NF_BAIXA_AF';
    const PROCEDURE_DELETA_NF               = 'DELETA_NF';
    
    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
        $this->entityManager = $entityManager;
    }
    
    public function itemProduto($filial,$lancamento){
        $sql = "SELECT *
            FROM ITEMPRODUTO
            WHERE CDFILIAL = :P_CDFILIAL
            AND NRLANCTONF  = :P_NRLANCTONF";
		       
		       
		 $params = array(
		     "P_CDFILIAL"		=> $filial,
             "P_NRLANCTONF"      => $lancamento
        );
                        
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
    }

    public function validaCDFILIAL($cdfilial){
        $sql =  "SELECT CDEMPRESA, CDINSCESTA, CDINSCMUNI FROM FILIAL WHERE CDFILIAL = :CDFILIAL ";
               
        $params = array(
            "CDFILIAL" => $cdfilial
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $filialWasFound = $query->fetch();
        if(isset($filialWasFound) && $filialWasFound != false){
            return $filialWasFound;
        }else{
            return false;
        }
    }
    
    public function validaAF($nrautoforn){
        $sql =  "SELECT * FROM autoforne WHERE NRAUTOFORN = :NRAUTOFORN AND NVL(IDBAIXAAF, '0') <> '2'";
               
        $params = array(
            "NRAUTOFORN" => $nrautoforn
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $afWasFound = $query->fetch();
        if(isset($afWasFound) && $afWasFound != false){
            return true;
        }else{
            return false;
        }
    }
    
    public function validaCDTIPOOPER($cdtipooper, $identrsaiop){
        //$sql =  "SELECT NVL(IDTRANSMIFEBOS , 'N') FROM TIPOOPERAC WHERE CDTIPOOPER = :CDTIPOOPER AND IDENTRSAIDOP = :IDENTRSAIDOP";
        $sql =  "SELECT * FROM TIPOOPERAC WHERE CDTIPOOPER = :CDTIPOOPER AND IDENTRSAIDOP = :IDENTRSAIDOP";
               
        $params = array(
            "CDTIPOOPER" => $cdtipooper,
            "IDENTRSAIDOP" => $identrsaiop
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $tipoperWasFound = $query->fetch();
        if($tipoperWasFound != false && isset($tipoperWasFound)){
             return $tipoperWasFound["IDTRANSMIFEBOS"];
        }else{
            return false;
        }
    }
    
    public function validaNRNOTAFISC($nrnotafisc, $cdforneced){
        $sql =  "select nrnotafisc, cdforneced from notafiscal where nrnotafisc = :nrnotafisc  and cdforneced = :cdforneced";
               
        $params = array(
            "nrnotafisc" => $nrnotafisc,
            "cdforneced" => $cdforneced
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $nrWasFound = $query->fetch();
        if($nrWasFound != false && isset($nrWasFound)){
             return true;
        }else{
            return false;
        }
    }

    public function validaCDFORNECED($nrinsjurforn){
        $sql =  "SELECT CDFORNECED from fornecedor WHERE LPAD(NRINSJURFORN, 20, '0') = LPAD(:NRINSJURFORN, 20, '0')";
               
        $params = array(
            "NRINSJURFORN" => $nrinsjurforn
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $fornWasFound = $query->fetch();
        if($fornWasFound != false && isset($fornWasFound)){
             return $fornWasFound;
        }else{
            return false;
        }
    }
    
    public function validaCDOPERADOR($cdoperador){
        $sql =  "SELECT CDOPERADOR FROM OPERADOR WHERE CDOPERADOR = :CDOPERADOR";
               
        $params = array(
            "CDOPERADOR" => $cdoperador
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $opWasFound = $query->fetch();
        if($opWasFound != false && isset($opWasFound)){
             return true;
        }else{
            return false;
        }
    }
    
    public function validaCDSERIEE($cdserie){
        $sql =  "select * from SERIEENTR where CDSERIEENTR = :CDSERIEENTR AND IDATIVO = 'S'";
               
        $params = array(
            "CDSERIEENTR" => $cdserie
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $serieWasFound = $query->fetch();
        if($serieWasFound != false && isset($serieWasFound)){
             return true;
        }else{
            return false;
        }
    }
    
    public function validaCDSERIES($cdserie, $cdempresa, $cdinscesta, $cdinscmuni){
        $sql =  "SELECT * FROM SERIENF WHERE CDSERIE = :CDSERIE AND CDEMPRESA = :CDEMPRESA AND IDATIVO = 'S' AND CDINSCESTA = :CDINSCESTA AND CDINSCMUNI = :CDINSCMUNI";
               
        $params = array(
            "CDSERIE" => $cdserie,
            "CDEMPRESA" => $cdempresa,
            "CDINSCESTA" => $cdinscesta,
            "CDINSCMUNI" => $cdinscmuni
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $serieWasFound = $query->fetch();
        if($serieWasFound != false && isset($serieWasFound)){
             return true;
        }else{
            return false;
        }
    }
    
    public function filialControlaEstoque($cdfilial){
        $sql =  "SELECT IDCTRLESTQ FROM PARAMFILIAL WHERE CDFILIAL = :CDFILIAL";
               
        $params = array(
            "CDFILIAL" => $cdfilial
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $controlaEstWasFound = $query->fetch();
        if(isset($controlaEstWasFound) && $controlaEstWasFound != false){
            return isset($controlaEstWasFound['IDCTRLESTQ']) ? $controlaEstWasFound['IDCTRLESTQ']: 'N';
        }else{
            return false;
        }
    }
    
    public function tipooperControlaEstoque($cdtipooper, $identrsaidop){
        $sql =  "SELECT IDATUESTTPOP FROM TIPOOPERAC WHERE CDTIPOOPER = :CDTIPOOPER AND IDENTRSAIDOP = :IDENTRSAIDOP";
               
        $params = array(
            "CDTIPOOPER" => $cdtipooper,
            "IDENTRSAIDOP" => $identrsaidop
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $tipooperacWasFound = $query->fetch();
        if(isset($tipooperacWasFound) && $tipooperacWasFound != false){
            return isset($tipooperacWasFound['IDATUESTTPOP']) ? $tipooperacWasFound['IDATUESTTPOP']: 'N';
        }else{
            return false;
        }
    }
    
    public function validateCharges($cdencargo){
        $sql =  "select NMENCARGO from encargo where CDENCARGO = :CDENCARGO AND IDATIVO = 'S'";
               
        $params = array(
            "CDENCARGO" => $cdencargo
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $encargoWasFound = $query->fetch();
        if(isset($encargoWasFound) && $encargoWasFound != false){
            return true;
        }else{
            return false;
        }
    }
    
    public function getInfosProducts($produto){
        $sql =  "SELECT CDARVPROD, CDPRODUTO, CDPRODESTO, NMPRODUTO FROM PRODUTO WHERE LPAD(CDPRODUTO, 20, '0') = LPAD(:CDPRODUTO, 20, '0') AND IDATIVO = 'S'";
               
        $params = array(
            "CDPRODUTO" => $produto
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $infos_prod = $query->fetch();
        if(isset($infos_prod) && $infos_prod != false){
            return $infos_prod;
        }else{
            return false;
        }
    }
    
    public function getInfoProdFili($cdprodesto, $cdfilial){
        $sql =  "SELECT IDCNTRESTOQ FROM PRODFILI WHERE CDFILIAL = :CDFILIAL AND CDPRODUTO = :CDPRODUTO AND IDATIVO = 'S'";
               
        $params = array(
            "CDPRODUTO" => $cdprodesto,
            "CDFILIAL" => $cdfilial
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $infos_prod = $query->fetch();
        if(isset($infos_prod) && $infos_prod != false){
            return $infos_prod;
        }else{
            return false;
        }
    }
    
    //entrada
    /*
    
        E - 01   
        cdfilial = cdfiliorig - OK
        $cdfilirece = cdfiliorig - OK
        $cdempresa = cdfilirece - OK 
        $cdtipooper - OK
        $cdserieentr - OK
        cdcliente = cdforneced - OK
        $sistema = DFE, 
        $atualizaestoque = OK
        $nrlanctonf -> NULL
        $nrlancestoq = NULL
        $dtlancmennf = sysdate
       
        E - 02
        cdfilial = cdfilidest - OK
        $cdfilirece = cdfilidest - OK
        $cdempresa = cdfilirece - OK
        $cdtipooper - OK
        $cdserieentr - OK
        cdcliente = cdfiliorig - OK
        $sistema = DFE, 
        $atualizaestoque = OK
        $nrlanctonf -> NULL
        $nrlancestoq = NULL
        $dtlancmennf = sysdate
    */
    public function geraCorpoNotaEntrada($cdfilial, $cdfilirece, $cdempresa, $cdtipooper, $cdserieentr, $cdcliente, $sistema, $atualizaestoque, 
        $nrlanctonf, $pesstrans, $insctrans, $placaveiculo, $ufveiculo, $pgtofrete, $quantidadedevolu, $espefvolu, $marcvolu, $numevolu, $nrlancestoq,
        $dtentrsaid, $vrbrutovolu, $vrliquidovolu, $vrnotafisc, $nrlancorig, $dtlancmennf, $dtemissao, $nrnotafisc, $nracessonfe, $cdmododocfisc) {
        
        $prxLanctoNf    = NULL;
        $prxLanctoEstoq = NULL;
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_NOTA_CORPO_ENTRADA);
        
        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE',      Param::PARAM_INPUT, $cdfilirece,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $cdempresa,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdtipooper,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERIEENTR',     Param::PARAM_INPUT, $cdserieentr,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cdcliente,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $dtentrsaid,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSISTEMA',       Param::PARAM_INPUT, $sistema,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRNOTAFISC',      Param::PARAM_INPUT, $vrnotafisc,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_PRXLANCTONF',     Param::PARAM_OUTPUT,$prxLanctoNf,       Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_PRXLANCESTQ',     Param::PARAM_OUTPUT,$prxLanctoEstoq,    Param::PARAM_TYPE_STR, 10));
        
        $procedure->addParam(new Param('P_ATUAESTOQUE',     Param::PARAM_INPUT, $atualizaestoque,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAPURACFAT',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, NULL,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDTPIJURTRAN',    Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURTRAN',    Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPLACAVEIC',     Param::PARAM_INPUT, NULL,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGUFVEIC',        Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPGTOFRETE',     Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        
        
        $procedure->addParam(new Param('P_DSQTDEVOLU',      Param::PARAM_INPUT, NULL,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSESPECVOLU',     Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSMARCAVOLU',     Param::PARAM_INPUT, NULL,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSNUMEVOLU',      Param::PARAM_INPUT, NULL,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPBRUTOVOLU',    Param::PARAM_INPUT, NULL,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPLIQUVOLU',     Param::PARAM_INPUT, NULL,     Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, NULL,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCORIG',      Param::PARAM_INPUT, NULL,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENNF',    Param::PARAM_INPUT, $dtlancmennf,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRNOTAFISC',      Param::PARAM_INPUT, $nrnotafisc,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTEMISSAO',       Param::PARAM_INPUT, $dtemissao,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRACESSONFE',     Param::PARAM_INPUT, NULL,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMODDOCFISC',    Param::PARAM_INPUT, NULL,     Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
        
    }
    
    public function insertDefaultCharges($cdfilial, $nrlanctonf){
        $sql =  "INSERT INTO ITEMENCARGO
                    (CDFILIAL, NRLANCTONF, CDENCARGO, VRITEMENCA)
                    (SELECT :CDFILIAL, :NRLANCTONF, CDENCARGO, 0
                       FROM ENCARGO
                      WHERE NOT EXISTS(SELECT CDENCARGO
                                         FROM ITEMENCARGO
                                        WHERE 
                                                CDFILIAL   = :CDFILIAL
                                            AND NRLANCTONF = :NRLANCTONF
                                            AND CDENCARGO  = ENCARGO.CDENCARGO))";
               
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function updateCharge($cdencargo, $vritemenca, $cdfilial, $nrlanctonf){
        $sql =  "UPDATE ITEMENCARGO
                   SET VRITEMENCA = :VRITEMENCA
                 WHERE CDENCARGO  = :CDENCARGO
                   AND CDFILIAL   = :CDFILIAL
                   AND NRLANCTONF = :NRLANCTONF";
               
        $params = array(
            "CDENCARGO" => $cdencargo,
            "VRITEMENCA" => $vritemenca,
            "CDFILIAL" => $cdfilial,
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function updateNf2Pending($cdfilial, $numLancamento, $status, $operador, $nrnotafisc){
        //var_dump($cdfilial, $numLancamento, $status, $operador, $nrnotafisc);
        $sql =  "UPDATE NOTAFISCAL
                   SET IDSTATUS = :IDSTATUS,
                       CDOPERGERA = :CDOPERGERA,
                       NRNOTAFISC = :NRNOTAFISC
                 WHERE
                       CDFILIAL   = :CDFILIAL
                   AND NRLANCTONF = :NRLANCTONF";
               
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRLANCTONF" => $numLancamento,
            "IDSTATUS" => $status, 
            "CDOPERGERA" => $operador,
            "NRNOTAFISC" => $nrnotafisc
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function taxApproved($cdfilial, $numLancamento){
        $sql =  "UPDATE NOTAFISCAL
                   SET IDBLOQUEADO = 'N',
                        IDSTATUS = 'A'
                 WHERE
                       CDFILIAL   = :CDFILIAL
                   AND NRLANCTONF = :NRLANCTONF";
               
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRLANCTONF" => $numLancamento
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function procedureConsultaGrupofiscal($idorigdestnf, $fornecedor, $filial, $entrada_saida, $tipoOperacao) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_CONSULTA_GRUPOFISCAL);
        $procedure->addParam(new Param("P_IDORIGDESTNF", Param::PARAM_INPUT, $idorigdestnf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDORIGDESTNF", Param::PARAM_INPUT, $fornecedor, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIFATUR", Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDENTRSAIDOP", Param::PARAM_INPUT, $entrada_saida, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDTIPOOPER", Param::PARAM_INPUT, $tipoOperacao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDGRUPOFISC", Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_STR, 10));
        $output = $procedure->execute();

        return $output['P_CDGRUPOFISC'];
    }
    
    public function procedureGeraFechamentoEntrada($filial, $numLancamento, $dtProcessamento, $numLancEstoque, $organizacao, $operador = ''){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $this->entityManager->flush();
        $connection->commit();
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_FECHAMENTO_ENTRADA);
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC', Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function vigenciaIE($ie, $emp, $dtEntrSaid){
        $sql = "SELECT CDINSCESTA
                  FROM INSCRESTAD
                 WHERE CDINSCESTA    = :P_CDINSCESTA
                   AND CDEMPRESA     = :P_CDEMPRESA
                   AND NVL(DTINIVIGIEST, :P_DTENTRSAID) <= :P_DTENTRSAID
                   AND NVL(DTFIMVIGIEST, :P_DTENTRSAID) >= :P_DTENTRSAID";
        
        $params = array(
            "P_CDINSCESTA"      => $ie,
            "P_CDEMPRESA"       => $emp,
            "P_DTENTRSAID"      => $dtEntrSaid
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();
    }
    
    public function geraCorpoNota($cdfilial, $cdfiliemit, $cdempresa, $cdtipooper, $cdserie, $cdcliente, $sistema, $atualizaestoque, 
        $nrlanctonf, $pesstrans, $insctrans, $placaveiculo, $ufveiculo, $pgtofrete, $quantidadedevolu, $espefvolu, $marcvolu, $numevolu, $nrlancestoq,
        $dtentrsaid, $vrbrutovolu, $vrliquidovolu, $nrlancorig = null, $dtlancmennf = null 
        ) {
        $dtentrsaid = str_replace("-", "/", $dtentrsaid);
        
        $prxLanctoNf    = NULL;
        $prxLanctoEstoq = NULL;
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_NOTA_CORPO);

        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT',      Param::PARAM_INPUT, $cdfiliemit,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $cdempresa,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdtipooper,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERIE',         Param::PARAM_INPUT, $cdserie,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cdcliente,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $dtentrsaid,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSISTEMA',       Param::PARAM_INPUT, $sistema,           Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_PRXLANCTONF',     Param::PARAM_OUTPUT,$prxLanctoNf,       Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_PRXLANCESTQ',     Param::PARAM_OUTPUT,$prxLanctoEstoq,    Param::PARAM_TYPE_STR, 10));
        
        $procedure->addParam(new Param('P_ATUAESTOQUE',     Param::PARAM_INPUT, $atualizaestoque,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAPURACFAT',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, NULL,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDTPIJURTRAN',    Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURTRAN',    Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPLACAVEIC',     Param::PARAM_INPUT, NULL,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGUFVEIC',        Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPGTOFRETE',     Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        
        
        $procedure->addParam(new Param('P_DSQTDEVOLU',      Param::PARAM_INPUT, NULL,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSESPECVOLU',     Param::PARAM_INPUT, NULL,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSMARCAVOLU',     Param::PARAM_INPUT, NULL,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSNUMEVOLU',      Param::PARAM_INPUT, NULL,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPBRUTOVOLU',    Param::PARAM_INPUT, NULL,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPLIQUVOLU',     Param::PARAM_INPUT, NULL,     Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, NULL,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCORIG',      Param::PARAM_INPUT, NULL,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENNF',    Param::PARAM_INPUT, $dtlancmennf,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDGEROUTDOCU',    Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTCOMPFATUR',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPTOVNDFATURNF', Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
    }
    
    public function procedureGeraFechamentoSaida($filial, $numLancamento, $dtProcessamento, $numLancEstoque, $operador = '', $evolEstoque = 'S', $bloqTrans = 'N'){
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_FECHAMENTO_SAIDA);
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC', Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_EVOLESTOQ', Param::PARAM_INPUT, $evolEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDBLOQTRANS', Param::PARAM_INPUT, $bloqTrans, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    //ambos
    public function procedureGeraNotaFiscalProduto(
        $filial, $numLancamento, $produto, $codEmissao, $descEmissao, $quantidade, $vrUnitario, $vrDesconto, 
        $almoxarifado = null, $localEstoque = null, $nrloteestq = null, $nrsublote = null, $cdservfatu = null,
        $idOrigemProd = null, $idAtuEstProd = 'S',  $vrICMSItOrig = null, $cdMarca = null, $cdservfili = null,
        $dsInfadicprod = null, $cdfisopeitem = null){
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_PRODUTO);
        
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTO', Param::PARAM_INPUT, $produto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODEMIS', Param::PARAM_INPUT, $codEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSPRODEMIS', Param::PARAM_INPUT, $descEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTITEMPROD', Param::PARAM_INPUT, $quantidade, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRUNITITEM', Param::PARAM_INPUT, $vrUnitario, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRDESCITEM', Param::PARAM_INPUT, $vrDesconto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDALMOXARIFE', Param::PARAM_INPUT, $almoxarifado, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOCALESTOQ', Param::PARAM_INPUT, $localEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOTEESTQ', Param::PARAM_INPUT, $nrloteestq, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSUBLOTE', Param::PARAM_INPUT, $nrsublote, Param::PARAM_TYPE_STR));
        

        $procedure->addParam(new Param('P_CDFILIALITG', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINTEGRACAO', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRITEMINTEGR', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCSTINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCFOPINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSUFXINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDCONFORITEM', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTPNFCREDEB', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTVOLUITEM', Param::PARAM_INPUT, 0, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAPUR', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFATU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGEMPROD', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDATUESTPROD', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRICMSSTITORIG', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSJUSTIFICATIVA', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSINFADICPROD', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFISOPEITEM', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    public function procedureImpostoEntradaProduto(
        $empresa, $inscesta, $tipoOperacao, $fornecedor, $filial,
        $seqLancamento, $dtEntrada, $entrada_saida, $operador, $grupofiscal,
        $idinterest, $cdarvprod, $cdgrpfiscsv, $idorigdestnf, $filialRece, $cdfiliemit,
        $cliente, $valorProduto, $sgEstado, $cdMunicipio
    ) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_IMPOSTO_ENTRADA_PRODUTO);
        $procedure->addParam(new Param('P_EMPRESA', Param::PARAM_INPUT, $empresa, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA', Param::PARAM_INPUT, $inscesta, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOOPER', Param::PARAM_INPUT, $tipoOperacao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, $fornecedor, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF', Param::PARAM_INPUT, $seqLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID', Param::PARAM_INPUT, $dtEntrada, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ENTRSAID', Param::PARAM_INPUT, $entrada_saida, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRUPOFISC', Param::PARAM_INPUT, $grupofiscal, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDINTEREST', Param::PARAM_INPUT, $idinterest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDARVPROD', Param::PARAM_INPUT, $cdarvprod, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRPFISCSV', Param::PARAM_INPUT, $cdgrpfiscsv, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGDESTNF', Param::PARAM_INPUT, $idorigdestnf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE', Param::PARAM_INPUT, $filialRece, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT', Param::PARAM_INPUT, $cdfiliemit, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $cliente, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPRODUTO', Param::PARAM_INPUT, $valorProduto, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDORIGEMPROD',    Param::PARAM_INPUT, 'N',  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGESTADO',        Param::PARAM_INPUT, $sgEstado,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMUNICIPIO',     Param::PARAM_INPUT, $cdMunicipio,   Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    public function getAliquota($cdfilial, $nrlanctonf){
        $sql =  "SELECT IM.CDIMPOSTO, IM.SGIMPOSTO, IM.IDTRATIMPO, IM.IDPREIMPENTR,
                        I.CDFISCTEMP, NVL(AL.VRALIQUTMP, 0) AS VRALIQUOTA,
                        NVL(AL.VRADICALIQFECP, 0) AS VRADICALIQFECP
                   FROM INCITEMP I, IMPOSTO IM, ALIQTEMPPROD AL
                  WHERE I.CDIMPOTEMP  = IM.CDIMPOSTO
                    AND I.NRINCITEMP  = :CDFILIAL||:NRLANCTONF
                    AND I.NRINCITEMP  = AL.NRALIQTEMP(+)
                    AND I.CDIMPOTEMP  = AL.CDIMPOTEMP(+)
                    AND IM.IDPRODNOTA <> 'C'";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $aliquotaWasFound = $query->fetchAll();
        if(!empty($aliquotaWasFound) && isset($aliquotaWasFound)){
            return $aliquotaWasFound;
        }else{
            return false;
        }
    }
    
    public function insertTrabitproimpos($cdfilial, $nrlanctonf, $imposto, $produto){
        $sql =  "INSERT INTO TRABITPROIMPOS
                    (CDFILIMPOIT, NRLANCIMPOIT, NRSEQIMPOIT, CDIMPOSTOIT, VRCONTIMPOIT,
                    VRREDBCIMPOIT, VRBCALCIMPOIT, VRALIQIMPOIT, VRPDEDIMPOIT, VRIMPOSIT,
                    CDFISCIMPOIT,  NRSEQITPRO, IDVALORINFO,
                    VR1REDBCIMPOIT, IDTRIMPSTTI, VRIMPOSVISUIT, VRADICALIQFECP)
                 VALUES
                    (:CDFILIAL, :NRLANCTONF, :NRSEQITPROD, :CDIMPOSTO, :VRTOTAITEMLIQUIDO,
                     0, :VRTOTAITEM, :VRALIQUOTA, 0, 0,
                     :CDFISCTEMP, '1', 'N',
                     0, '0', 0, :VRADICALIQFECP)";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf,
            "NRSEQITPROD" => $produto["NRSEQITPROD"],
            "CDIMPOSTO" => $imposto["CDIMPOSTO"],
            "VRTOTAITEMLIQUIDO" => ($produto["VRUNITITEM"] * $produto["QTITEMPROD"]) - $produto["VRDESCITEM"],
            "VRTOTAITEM" => ($produto["VRUNITITEM"] * $produto["QTITEMPROD"]),
            "VRALIQUOTA" => $imposto["VRALIQUOTA"],
            "CDFISCTEMP" => $imposto["CDFISCTEMP"],
            "VRADICALIQFECP" =>$imposto["VRADICALIQFECP"]
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function getMaxNrSeqItProd($cdfilial, $nrlanctonf){
        $sql =  "SELECT LPAD(MAX(NRSEQITPROD), 4, '0') AS NRSEQITPROD FROM ITEMPRODUTO WHERE CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $maxWasFound = $query->fetch();
        if($maxWasFound["NRSEQITPROD"] == null){
            return '0001';
        }else{
            return $maxWasFound["NRSEQITPROD"];
        }
    }
    
    public function getIdinterest($empresa, $cdinscesta, $cliente, $fornecedor, $filial, $idorigdestnf, $organizacao) {
        $idinterest = 'N';

        $endeEmpresa = Empresa::getRepository()->getEstadoPais(
            $empresa, $cdinscesta, $organizacao
        );

        switch ($idorigdestnf) {
            case 'F':
                $endeForn = Endeforn::getRepository()->getEstadoPais($fornecedor, $organizacao);
                if ($endeEmpresa['CDPAIS'] == $endeForn['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeForn['SGESTADO']) {
                    $idinterest = 'S';
                }
                break;
            case 'C':
                $endeClie = Endeclie::getRepository()->getEstadoPais($cliente, $organizacao);
                if ($endeEmpresa['CDPAIS'] == $endeClie['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeClie['SGESTADO']) {
                    $idinterest = 'S';
                }
                break;
            case 'L':
                $endeFili = Endefili::getRepository()->getEstadoPais($filial, $organizacao);
                if ($endeEmpresa['CDPAIS'] == $endeFili['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeFili['SGESTADO']) {
                    $idinterest = 'S';
                }
                break;
        }
        return $idinterest;
    }

    public function getTrabImposNfTot($cdfilial, $nrlanctonf){
        $sql =  "SELECT SUM(T.VRIMPOSIT) VRTOTAIMPO, T.CDIMPOSTOIT
                    FROM TRABITPROIMPOS T, IMPOSTO I
                   WHERE CDFILIMPOIT  = :CDFILIAL
                     AND NRLANCIMPOIT = :NRLANCTONF
                     AND IDPRODNOTA <> 'C'
                     AND T.CDIMPOSTOIT = I.CDIMPOSTO
                   GROUP BY CDIMPOSTOIT
                   ORDER BY CDIMPOSTOIT";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $impostoWasFound = $query->fetchAll();
        if(!empty($impostoWasFound) && isset($impostoWasFound)){
            return $impostoWasFound;
        }else{
            return false;
        }
    }
    
    public function insertTrabImposNfTot($cdfilial, $nrlanctonf, $imposto){
        $sql =  "INSERT INTO TRABIMPOSNFTOT
                        (CDFILIMPTOT, 
                        NRLANCIMPTOT, 
                        CDIMPOSTOT, 
                        VRIMPOSTOT, 
                        VRBCALIMPTOT)
                  VALUES
                        (:CDFILIAL, 
                        :NRLANCTONF, 
                        :CDIMPOSTOIT, 
                        (0 + :VRTOTAIMPO), 
                        0)";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf,
            "CDIMPOSTOIT" => $imposto["CDIMPOSTOIT"],
            "VRTOTAIMPO" => $imposto["VRTOTAIMPO"]
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function procedureFechamento($tax){
        if($tax['IDENTRSAIDOP'] == 'S'){
            $this->procedureGeraFechamentoSaida(
                $tax['CDFILIAL'], $tax['NRLANCTONF'], $tax["DTPROCESSA"], $tax['NRLANCESTQ'], $tax['CDOPERADOR']
            );
        } else{
            $this->procedureGeraFechamentoEntrada(
                $tax['CDFILIAL'], $tax['NRLANCTONF'], $tax["DTPROCESSA"], $tax['NRLANCESTQ'], $tax["NRORG"], $tax['CDOPERADOR']
            );
        }
        $this->entityManager->flush();
    }
    
    public function getDTPROCESSA($cdfilial, $nrorg){
       $sql =  "SELECT DTPROCESSA FROM paramfilial where cdfilial = :cdfilial AND NRORG = :NRORG";
               
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRORG" => $nrorg
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $PFWasFound = $query->fetch();
        if(isset($PFWasFound) && $PFWasFound != false){
            return $PFWasFound["DTPROCESSA"];
        }else{
            return false;
        }
    }
    
    public function getMovilote($nrmovilote, $nrorg){
       $sql =  "select * from Movilote WHERE NRMOVILOTE = :NRMOVILOTE AND NRORG = :NRORG";
               
        $params = array(
            "NRMOVILOTE" => $nrmovilote,
            "NRORG" => $nrorg
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $moviWasFound = $query->fetchAll();
        if(isset($moviWasFound) && !empty($moviWasFound)){
            return $moviWasFound;
        }else{
            return false;
        }
    }
    
    public function getLanctoEstoq($cdfilial, $nrlancestoq){
       $sql =  "select * from lanctoestoq where CDFILIAL = :CDFILIAL AND NRLANCESTQ = :NRLANCESTQ";
               
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRLANCESTQ" => $nrlancestoq
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $lanctoEstoqWasFound = $query->fetch();
        if(isset($lanctoEstoqWasFound) && $lanctoEstoqWasFound != false){
            return $lanctoEstoqWasFound;
        }else{
            return false;
        }
    }
    
    public function getNf($cdfilial, $nrlanctonf){
       $sql =  "select * from NOTAFISCAL where CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF";
               
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $nfWasFound = $query->fetch();
        if(isset($nfWasFound) && $nfWasFound != false){
            return $nfWasFound;
        }else{
            return false;
        }
    }
    
    public function getItemproimpos($cdfilial, $nrlanctonf){
       $sql =  "select * from itemproimpos WHERE NRLANCTONF = :NRLANCTONF AND CDFILIAL = :CDFILIAL";
               
        $params = array(
            "NRLANCTONF" => $nrlanctonf,
            "CDFILIAL" => $cdfilial
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $impostoWasFound = $query->fetchAll();
        if(isset($impostoWasFound) && !empty($impostoWasFound)){
            return $impostoWasFound;
        }else{
            return false;
        }
    }
    
    public function getNfTot($cdfilial, $nrlanctonf){
        $sql =  "select * from Imposnftot WHERE NRLANCTONF = :NRLANCTONF AND CDFILIAL = :CDFILIAL";
               
        $params = array(
            "NRLANCTONF" => $nrlanctonf,
            "CDFILIAL" => $cdfilial
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $impostoWasFound = $query->fetchAll();
        if(isset($impostoWasFound) && !empty($impostoWasFound)){
            return $impostoWasFound;
        }else{
            return false;
        }
    }
    
    public function getProductOneByCdprodutoAndNrorg($CDPRODUTO, $NRORG){
       $sql =  "select * from produto WHERE CDPRODUTO = :CDPRODUTO AND NRORG = :NRORG";
               
        $params = array(
            "CDPRODUTO" => $CDPRODUTO,
            "NRORG" => $NRORG
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $impostoWasFound = $query->fetch();
        if(isset($impostoWasFound) && $impostoWasFound != false){
            return $impostoWasFound;
        }else{
            return false;
        }
    }

    public function deleteAliqtempprod($cdfilial, $nrlanctonf){
        $sql =  "DELETE
                 FROM ALIQTEMPPROD
                WHERE NRALIQTEMP = :CDFILIAL||:NRLANCTONF";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function deleteIncitemp($cdfilial, $nrlanctonf){
        $sql =  "DELETE
                 FROM INCITEMP
                WHERE NRINCITEMP = :CDFILIAL||:NRLANCTONF";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function getAf($nrautoforn, $cdproduto){
        $sql =  "SELECT ITEMAUTF.QTPROGENTAF, ITEMAUTF.QTEFETENTRAF, ITEMAUTF.IDBAIXITMAF, ITEMAUTF.VRPREUNIAF,
                       ITEMAUTF.* 
                  FROM ITEMAUTF
                 WHERE NRAUTOFORN = :NRAUTOFORN  
                 AND CDPRODUTO = :CDPRODUTO";
               
        $params = array(
            "NRAUTOFORN" => $nrautoforn,
            "CDPRODUTO" => $cdproduto
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $afWasFound = $query->fetch();
        if(isset($afWasFound) && $afWasFound != false){
            return $afWasFound;
        }else{
            return false;
        }
    }
    
    public function insertNfRelacAf($cdfilial, $CDFILLANC, $nrlanctonf,
                                    $CDFILDEST, $nrautoforn, $CDPRODUTO, 
                                    $CDMARCA, $DTPROGENTAF, $QTPROGENTAF,
                                    $VRPREUNIAF){
        $sql =  "   INSERT INTO NFRELACAF
                    (CDFILIAL, CDFILLANC, NRLANCTONF, CDFILDEST,
                     NRAUTOFORN, CDPRODUTO, CDMARCA,
                     DTPROGENTAF, QTNFRELACAF, VRNFRELACAF)
                    VALUES
                    (:CDFILIAL, :CDFILLANC, :NRLANCTONF, :CDFILDEST,
                     :NRAUTOFORN, :CDPRODUTO, :CDMARCA,
                    :DTPROGENTAF, :QTPROGENTAF, :VRPREUNIAF)";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf,
            "CDFILLANC" => $CDFILLANC, 
            "CDFILDEST" => $CDFILDEST,
            "NRAUTOFORN" => $nrautoforn, 
            "CDPRODUTO" => $CDPRODUTO, 
            "CDMARCA" => $CDMARCA,
            "DTPROGENTAF" => $DTPROGENTAF, 
            "QTPROGENTAF" => $QTPROGENTAF, 
            "VRPREUNIAF" => $VRPREUNIAF
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function updateMarcaAf($cdmarca, $cdfilial, $nrlanctonf, $nrseqitprod, $dtprogentaf, $nrautoforn, $cdfildest){
        $sql =  "UPDATE ITEMPRODUTO
                      SET CDMARCA     = :CDMARCA,
                          DTPROGENTAF = :DTPROGENTAF,
                          NRAUTOFORN  = :NRAUTOFORN,
                          CDFILDEST   = :CDFILDEST
                    WHERE CDFILIAL    = :CDFILIAL
                      AND NRLANCTONF  = :NRLANCTONF
                      AND NRSEQITPROD = LPAD(:NRSEQITPROD, 4, '0')";
               
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRLANCTONF" => $nrlanctonf,
            "CDMARCA" => $cdmarca,
            "NRSEQITPROD" => $nrseqitprod,
            "DTPROGENTAF" => $dtprogentaf,
            "NRAUTOFORN" => $nrautoforn,
            "CDFILDEST" => $cdfildest
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function updateQtRecebe($cdfilial, $nrlanctonf, $nrseqitprod, $qtrecebe){
        //var_dump($cdfilial, $nrlanctonf, $nrseqitprod, $qtrecebe);
        $sql =  "UPDATE ITEMPRODUTO
                      SET QTITEMRECEPDA = :QTRECEBE
                    WHERE CDFILIAL    = :CDFILIAL
                      AND NRLANCTONF  = :NRLANCTONF
                      AND NRSEQITPROD = LPAD(:NRSEQITPROD, 4, '0')";
               
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRLANCTONF" => $nrlanctonf,
            "NRSEQITPROD" => $nrseqitprod,
            "QTRECEBE" => $qtrecebe
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function procedureBaixaAF($cdfilial, $nalanctonf, $nrseqitprod, $idstatus, $newCdprodemis, $oldCdprodemis, $newCdproduto, $oldCdproduto, $newMarca, $oldMarca, $newQtitemprod, $oldQtitemprod) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_NF_BAIXA_AF);
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nalanctonf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQITPROD', Param::PARAM_INPUT, $nrseqitprod, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSTATUS', Param::PARAM_INPUT, $idstatus, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NEWCDPRODEMIS', Param::PARAM_INPUT, $newCdprodemis, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OLDCDPRODEMIS', Param::PARAM_INPUT, $oldCdprodemis, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NEWCDPRODUTO', Param::PARAM_INPUT, $newCdproduto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OLDCDPRODUTO', Param::PARAM_INPUT, $oldCdproduto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NEWCDMARCA', Param::PARAM_INPUT, $newMarca, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OLDCDMARCA', Param::PARAM_INPUT, $oldMarca, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NEWQTITEMPROD', Param::PARAM_INPUT, $newQtitemprod, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OLDQTITEMPROD', Param::PARAM_INPUT, $oldQtitemprod, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDERRO', Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_STR, 1));
        $procedure->execute();
    }
    
    public function procedureDeletaNf($filial, $lancamento, $corpoItem = null){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_DELETA_NF);
        $procedure->addParam(new Param('P_FILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF', Param::PARAM_INPUT, $lancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDCORPITEM', Param::PARAM_INPUT, $corpoItem, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function getProducts($cdfilial, $nrlanctonf){
        $sql =  "SELECT * FROM ITEMPRODUTO WHERE CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF";
               
        $params = array(
            "NRLANCTONF" => $nrlanctonf,
            "CDFILIAL" => $cdfilial
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $prodWasFound = $query->fetchAll();
        if(isset($prodWasFound) && !empty($prodWasFound)){
            return $prodWasFound;
        }else{
            return false;
        }
    }
    
    public function getProductsDevo($cdfilial, $nrlanctonf){
        $sql = "SELECT CDPRODUTO, CDPRODEMIS AS CDARVPROD, DSPRODEMIS, 
                       (QTITEMPROD - NVL(QTITEMRECEPDA, QTITEMPROD)) AS QTITEMPROD, 
                       VRUNITITEM, VRDESCITEM, CDALMOXARIFE, CDLOCALESTOQ,
                       NRLOTEESTQ, NRSUBLOTE, IDATUESTPROD, NRAUTOFORN
                  FROM ITEMPRODUTO 
                 WHERE CDFILIAL = :CDFILIAL 
                   AND NRLANCTONF = :NRLANCTONF 
                   AND QTITEMPROD > NVL(QTITEMRECEPDA, QTITEMPROD)";
               
        $params = array(
            "NRLANCTONF" => $nrlanctonf,
            "CDFILIAL" => $cdfilial
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $prodWasFound = $query->fetchAll();
        if(isset($prodWasFound) && !empty($prodWasFound)){
            return $prodWasFound;
        }else{
            return false;
        }
    }
    
    public function getTipoOperDevolucao(){
        $sql =  "select MIN(CDTIPOOPER) AS CDTIPOOPER from tipooperac T where IDENTRSAIDOP = 'S' AND IDOPERDEVONF = 'S' and IDORIGDESTOP = 'F'";
               
        $params = array();

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $tipooperWasFound = $query->fetch();
        if(isset($tipooperWasFound) && $tipooperWasFound != false){
            return $tipooperWasFound["CDTIPOOPER"];
        }else{
            return false;
        }
    }

    public function insertItemNotaFisc($cdfilialit, $nrlanctoit, $cdfilialdevo, $nrlanctonfdevo){
        $sql =  "INSERT INTO ITEMNOTAFISC
                    (CDFILIAL, NRLANCTONF, NRSEQUITNOTA, CDFILIALIT, NRLANCTONFIT)
                 VALUES
                    (:CDFILIAL, :NRLANCTONF, '001', :CDFILIALIT, :NRLANCTONFIT)";
               
        $params = array(
            "CDFILIAL" => $cdfilialdevo, 
            "NRLANCTONF" => $nrlanctonfdevo,
            "CDFILIALIT" => $cdfilialit,
            "NRLANCTONFIT" => $nrlanctoit
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function insertAdditionalData($dac, $cdfilial, $nrlanctonf, $nrdadoadic){
        $sql =  "INSERT INTO DADOSADIC
                        (CDFILIAL, 
                        NRLANCTONF, 
                        NRDADOADIC,
                        DSDADOADIC)
                  VALUES
                        (:CDFILIAL, 
                        :NRLANCTONF, 
                        :NRDADOADIC,
                        :DSDADOADIC)";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf,
            "NRDADOADIC" => $nrdadoadic,
            "DSDADOADIC" => $dac
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function getMaxNrDadoAdic($cdfilial, $nrlanctonf){
        $sql =  "SELECT LPAD(MAX(NRDADOADIC)+ '1', 3, '0') AS NRDADOADIC FROM DADOSADIC WHERE CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $maxWasFound = $query->fetch();
        if($maxWasFound["NRDADOADIC"] == null){
            return '001';
        }else{
            return $maxWasFound["NRDADOADIC"];
        }
    }
    
    public function getSeqAnexo($cdfilial, $nrlanctonf){
        $sql =  "SELECT LPAD(MAX(NRSEQANEXO) + '1', 4, '0') AS NRSEQANEXO FROM ANEXONFE WHERE CDFILIAL = :CDFILIAL AND NRLANCTONF = :NRLANCTONF";
               
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $maxWasFound = $query->fetch();
        if($maxWasFound["NRSEQANEXO"] == null){
            return '0001';
        }else{
            return $maxWasFound["NRSEQANEXO"];
        }
    }
    
    public function insertAnexo($cdfilial, $nrlanctonf, $dslinkanexo, $nrseqanexo){
        $sql =  "INSERT INTO ANEXONFE
                                    (CDFILIAL, NRLANCTONF, DSLINKANEXO, NRSEQANEXO)
                                 VALUES
                                    (:CDFILIAL, :NRLANCTONF, :DSLINKANEXO, :NRSEQANEXO)";
        $params = array(
            "CDFILIAL" => $cdfilial, 
            "NRLANCTONF" => $nrlanctonf,
            "DSLINKANEXO" => $dslinkanexo,
            "NRSEQANEXO" => $nrseqanexo
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
}
